/*
 *  Copyright
 */

/************************

INCLUDE

************************/

#include <iostream>
#include "hppOpenHRP/parserOpenHRPKineoObstacle.h" 


/************************

METHODS

************************/

//=========================================================================

CparserOpenHRPKineoObstacle::CparserOpenHRPKineoObstacle(std::vector<ModelInfo_var> inObstInfoVector){

  attObstInfoVector = inObstInfoVector ;
 
}


//=========================================================================

CparserOpenHRPKineoObstacle::~CparserOpenHRPKineoObstacle(){
}


//=========================================================================

vector<CkppKCDPolyhedronShPtr> CparserOpenHRPKineoObstacle::parser() {

  vector<CkppKCDPolyhedronShPtr> kineoObstacleModelVector ;

  for(unsigned int i=0; i < attObstInfoVector.size(); i++){

   
    TopenHrpJoint openHRPObstModel = (*(attObstInfoVector[i]->getCharObject()->modelObjectSeq()))[0] ;
   
    CkppKCDPolyhedronShPtr kineoObstacleModel = buildKppObstacle(openHRPObstModel, i) ;
    
    if (kineoObstacleModel)
      kineoObstacleModelVector.push_back(kineoObstacleModel) ;
    else {
      cerr << " ERROR CparserOpenHRPKineoObstacle::parser() : Failed to build the "<< i <<" obstacle" << endl ; 
      kineoObstacleModelVector.clear() ;
    }
    
  }
  
  return kineoObstacleModelVector ;
}

// ==============================================================================

CkppKCDPolyhedronShPtr CparserOpenHRPKineoObstacle::buildKppObstacle(const TopenHrpJoint& openHRPObstModel, unsigned int obstId ) 
{
  CkppKCDPolyhedronShPtr kineoObstacleModel ;
  kineoObstacleModel = buildHppPolyhedron(
    attObstInfoVector[obstId], openHRPObstModel,
    attObstInfoVector[obstId]->getCharObject()->name());
  if (!kineoObstacleModel) return kineoObstacleModel;

  ValueMatrix *relPos;
  ValueMatrix *relAttitude;
  double jointValue;
  CkitMat4 kwsRelPosMat;
  CkitMat4 *jointAbsPosMatPtr, *kwsAbsPosParentMatPtr=NULL;
    
  long parentId = openHRPObstModel->mother();
  if(parentId < 0) {
    // Joint is root joint, set position matrix of parent joint to Identity.
    kwsAbsPosParentMatPtr = new CkitMat4; 
    kwsAbsPosParentMatPtr->identity();
  }
  else
    std::cerr<<" obstacle parentId "<<parentId<<std::endl;
    
  relPos = openHRPObstModel->relPos();
  relAttitude = openHRPObstModel->relAttitude();
  jointValue = openHRPObstModel->jointValue();
  if (jointValue != 0) {
    cerr << "ERROR CparserOpenHRPKineoObstacle::buildKppObstacle : Joint value should be 0 " << endl;
    kineoObstacleModel.reset() ;
    return kineoObstacleModel;
  }

  // Get values of rotation matrix from relative position and attitude.
  getPositionMatrix(relPos, relAttitude, kwsRelPosMat);
      
  // Absolute position of current joint = absolute position parent joint * relative position current joint.
  jointAbsPosMatPtr = new CkitMat4((*kwsAbsPosParentMatPtr)*kwsRelPosMat);
    
  kineoObstacleModel->setAbsolutePosition(*jointAbsPosMatPtr);
    
  // Important - make the KCD collisionEntity!!
  kineoObstacleModel->makeCollisionEntity();
    
    
  return kineoObstacleModel;
};

